#include "bezier.h"



/* The first and last segments are not drawn--the first and last point are in
 * the list only for purposes of calculation of other segments.  Note that you
 * can make two copies of the first point at the head of the list and two copies
 * of the last point at the tail if you want the segments to be drawn */
void draw_bezier(GdkDrawable *drawable, GdkGC *gc, GdkPoint *points, unsigned gran)
{
    int       t, t_sq, t_cube, olt, x, y, xlast, ylast;
    long      gran_cube;
    GdkPoint *p1, *p2, *p3, *p4;

    gran_cube = gran * gran * gran;
    p1 = points;
    p2 = points + 1;
    p3 = points + 2;
    p4 = points + 3;

    xlast = p1->x; /* t == 0 */
    ylast = p1->y; /* t == 0 */

    for ( t = 0; t <= gran; t++ ) {
        /*
         * The formula is:
         *
         * $$ z(t) = (1 - t)^3 z_1 + 3(1 - t)^2 t z_2 +
         *           3(1 - t) t^2 z_3 + t^3 z_4 $$
         */
        t_sq = t * t;
        t_cube = t_sq * t;
        olt = gran - t;

        x = (((olt*p1->x + 3*t*p2->x)*olt + 3*t_sq*p3->x)*olt + p4->x*t_cube) / gran_cube;
        y = (((olt*p1->y + 3*t*p2->y)*olt + 3*t_sq*p3->y)*olt + p4->y*t_cube) / gran_cube;

        gdk_draw_line( drawable, gc, xlast, ylast, x, y );
        xlast = x;
        ylast = y;

    }

    return;
}



/*
 * Local variables:
 * compile-command: "make -k && ./bezier"
 * End:
 */
