#include "ball.h"
#include <math.h>


void circle( GdkDrawable *drawable, GdkGC *gc,
             gint xc, gint yc, gint R );
void circle_double( GdkDrawable *drawable, GdkGC *gc,
                    gint xc, gint yc, gint R );


void draw_ball( GdkDrawable *drawable, GdkColor *palette,
                guint num_shades, guint Ka, guint Kd, guint Ks,
                gint xc, gint yc, guint R )
{
    GdkGC *gc;
    gint   x, I;
    glong  xsq, Rsq;
    double Id, Is;

    assert( Ka + Kd + Ks < num_shades );

    gc = gdk_gc_new((GdkWindow *) drawable);
    Rsq = R*R;
    x   = R;
    xsq = Rsq;

    /* We move from outer radius down to radius of 1, drawing circles with
     * calculated intensity */
    for ( ; x > 0; xsq -= x+x - 1, x-- ) {
        /* Diffused light intensity */
        Id = sqrt(Rsq - xsq) / R;
        /* Reflected light intensity (ball surface's luster) */
        Is = (x <= R * M_SQRT1_2) ? pow(2*(Rsq-xsq)/(double)Rsq-1, 4) : 0;
        /* Total intensity--we take Ia (dispersed light intensity) to be 1, so
         * it is controlled entirely by Ka */
        I  = Ka + Kd*Id + Ks*Is;

        gdk_gc_set_foreground(gc, palette + I);
        circle_double(drawable, gc, xc, yc, x);
    }

    gdk_gc_unref(gc);
    return;
}


/*
 * Local variables:
 * compile-command: "make -k && ./ball"
 * End:
 */
