#include "ball.h"

#include "red.xpm"
#include "green.xpm"
#include "blue.xpm"


/* Ka, Ks, and Kd are coefficients for the three lighting intensities (see
 * draw_ball()) */
guint      Ka = DEFAULT_KA, Ks = DEFAULT_KS, Kd = DEFAULT_KD;
/* Number of shades in the palette and palette */
guint      num_shades = DEFAULT_NUM_SHADES;
GdkColor  *palette = NULL;
/* r, g, and b are intensity percentages of red, green, and blue colors */
gulong     red = DEFAULT_RED, green = DEFAULT_GREEN, blue = DEFAULT_BLUE;

/* Widget ID's and widgets */
enum{
    KA_WIDGET, KD_WIDGET, KS_WIDGET, SHADES_WIDGET,
        RED_WIDGET, GREEN_WIDGET, BLUE_WIDGET
};
static GtkWidget *widget_Ka, *widget_Ks, *widget_Kd;
static GtkWidget *widget_red,  *widget_green,  *widget_blue;
static GtkWidget *widget_shades;


static int setup_palette(void);


void adj_changed(GtkWidget *widget, gpointer sw)
{
    gboolean change_palette = TRUE, queue_resize = TRUE;

    switch ( (int) sw ) {
    case KA_WIDGET: {
        gint ka;

        ka = (gint) gtk_range_get_adjustment(GTK_RANGE(widget_Ka))->value;
        if ( ka + Kd + Ks >= num_shades )  /* value over the limit */
            ka = num_shades - Kd - Ks - 1; /* give maximum possible */
        if ( ka == Ka ) /* fell back to current value, no need to redraw */
            queue_resize = FALSE;
        Ka = ka;
        /* Even if `ka == Ka' above, value contained in the widget can still be
         * different from Ka (user jumped the limit).  But then, try to set
         * adjustment's value to Ka only if it is really different, just in case
         * it might send unwanted `changed' signal */
        if ( Ka != (gint) gtk_range_get_adjustment(GTK_RANGE(widget_Ka))->value )
            gtk_adjustment_set_value(
                gtk_range_get_adjustment(GTK_RANGE(widget_Ka)), Ka );

        change_palette = FALSE; /* don't touch palette, only redraw picture */
        break;
    }
    case KD_WIDGET: {
        gint kd;

        kd = (gint) gtk_range_get_adjustment(GTK_RANGE(widget_Kd))->value;
        if ( Ka + kd + Ks >= num_shades )  /* value over the limit */
            kd = num_shades - Ka - Ks - 1; /* give maximum possible */
        if ( kd == Kd ) /* fell back to current value, no need to redraw */
            queue_resize = FALSE;
        Kd = kd;
        /* Even if `kd == Kd' above, value contained in the widget can still be
         * different from Kd (user jumped the limit).  But then, try to set
         * adjustment's value to Kd only if it is really different, just in case
         * it might send unwanted `changed' signal */
        if ( Kd != (gint) gtk_range_get_adjustment(GTK_RANGE(widget_Kd))->value )
            gtk_adjustment_set_value(
                gtk_range_get_adjustment(GTK_RANGE(widget_Kd)), Kd );

        change_palette = FALSE; /* don't touch palette, only redraw picture */
        break;
    }
    case KS_WIDGET: {
        gint ks;

        ks = (gint) gtk_range_get_adjustment(GTK_RANGE(widget_Ks))->value;
        if ( Ka + Kd + ks >= num_shades )  /* value over the limit */
            ks = num_shades - Ka - Kd - 1; /* give maximum possible */
        if ( ks == Ks ) /* fell back to current value, no need to redraw */
            queue_resize = FALSE;
        Ks = ks;
        /* Even if `ks == Ks' above, value contained in the widget can still be
         * different from Ks (user jumped the limit).  But then, try to set
         * adjustment's value to Ks only if it is really different, just in case
         * it might send unwanted `changed' signal */
        if ( Ks != (gint) gtk_range_get_adjustment(GTK_RANGE(widget_Ks))->value )
            gtk_adjustment_set_value(
                gtk_range_get_adjustment(GTK_RANGE(widget_Ks)), Ks );

        change_palette = FALSE; /* don't touch palette, only redraw picture */
        break;
    }

    case SHADES_WIDGET: {
        gint ka, kd, ks, ksum, shades;

        ka = (gint) gtk_range_get_adjustment(GTK_RANGE(widget_Ka))->value;
        kd = (gint) gtk_range_get_adjustment(GTK_RANGE(widget_Kd))->value;
        ks = (gint) gtk_range_get_adjustment(GTK_RANGE(widget_Ks))->value;
        shades = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget_shades));
        ksum = ka + ks + kd;
        /* Here we decrement larger of the K* coefficients until the sum falls
         * into the limit */
        while ( ksum >= shades ) {
            if ( ka >= ks ) {
                if ( ka >= kd )
                    ka--;
                else
                    kd--;
            } else {
                if ( ks >= kd )
                    ks--;
                else
                    kd--;
            }
            ksum--;
        }
        num_shades = shades;

        /* Set new ranges for K* widgets */
        gtk_range_get_adjustment(GTK_RANGE(widget_Ka))->upper = num_shades - 1;
        gtk_range_get_adjustment(GTK_RANGE(widget_Kd))->upper = num_shades - 1;
        gtk_range_get_adjustment(GTK_RANGE(widget_Ks))->upper = num_shades - 1;

        /* Set new values for scale widgets.  Always set all K* before setting
         * any adjustment values--gtk_adjustment_set_value() sends
         * "value_changed" signals to widgets, and they should get all K*'s
         * correct, not just their own */
        Ka = ka;
        Kd = kd;
        Ks = ks;
        gtk_adjustment_set_value(
            gtk_range_get_adjustment(GTK_RANGE(widget_Ka)), Ka );
        gtk_adjustment_set_value(
            gtk_range_get_adjustment(GTK_RANGE(widget_Kd)), Kd );
        gtk_adjustment_set_value(
            gtk_range_get_adjustment(GTK_RANGE(widget_Ks)), Ks );

        break;
    }
    case RED_WIDGET:
        red =  (gint) gtk_range_get_adjustment(GTK_RANGE(widget_red))->value;
        break;
    case GREEN_WIDGET:
        green =  (gint) gtk_range_get_adjustment(GTK_RANGE(widget_green))->value;
        break;
    case BLUE_WIDGET:
        blue =  (gint) gtk_range_get_adjustment(GTK_RANGE(widget_blue))->value;
        break;
    default:
        assert(0);
        break;
    }

    if ( change_palette )
        setup_palette();
    /* Queue resize so that backing pixmap is created from scratch with the new
     * palette and/or Ka, Kd, and Ks coefficients */
    if ( queue_resize )
        gtk_widget_queue_resize(darea);

    return;
}

static int setup_palette(void)
{
    gulong  shade, step;
    int     i;

    assert( red   >= 0 && red   <= 100 && \
            green >= 0 && green <= 100 && \
            blue  >= 0 && blue  <= 100 );
    assert( num_shades > 1 );

    if ( palette )
        free( palette );

    /* Allocate buffer where we will store our palette */
    palette = (GdkColor *) malloc( sizeof(GdkColor) * num_shades );
    if ( !palette ) {
        fprintf(stderr, errMemory, progname);
        return 0;
    }

    step  = 0xffffl / (num_shades - 1);
    shade = 0;
    for ( i = 0; i < num_shades; i++, shade += step ) {
        /*  Set color from palette to the next shade of grey */
        (palette + i)->red   = (gushort) (shade * red   / 100);
        (palette + i)->green = (gushort) (shade * green / 100);
        (palette + i)->blue  = (gushort) (shade * blue  / 100);
        (palette + i)->pixel = ((palette + i)->red << 16) +
            ((palette + i)->green << 8) + (palette + i)->blue;

        if ( !gdk_color_alloc(gdk_window_get_colormap(main_window->window),
                              palette + i) ) {
            fprintf(stderr, errPalette, progname);
            return 0;
        }
    }

    return 1;
}

/* Construct hbox of label and spin button and show the widgets.  Returns the
 * constructed hbox.  If spin_ptr is not null, pointer to spin button is stored
 * in *spin_ptr.  Pass non-zero label_width and/or entry_width to set label
 * and/or spin button to those widths */
static GtkWidget *create_spin_button(gint min, gint max, gchar *label_text,
                                     gint set_val, gint page_increment,
                                     GtkWidget **widget_ptr,
                                     gint label_width, gint entry_width,
                                     gint widget_id)
{
    GtkWidget *adj, *widget;
    GtkWidget *box;

    box = gtk_hbox_new( FALSE, 0 );
    /* Label */
    widget = gtk_label_new(label_text);
    gtk_misc_set_alignment( GTK_MISC (widget), 0, .5 );
    gtk_widget_set_usize( widget, label_width, 0 );
    gtk_widget_show(widget);
    gtk_box_pack_start( GTK_BOX(box), widget, FALSE, FALSE, 5 );
    /* Spin button */
    adj = (GtkWidget *) gtk_adjustment_new(
        set_val, (gfloat) min, (gfloat) max,
        1.0, (gfloat) page_increment, 0.0 );
    widget = gtk_spin_button_new(
        GTK_ADJUSTMENT(adj), .5, 0 ); /* (climb_rate, digits) */
/*     gtk_spin_button_set_wrap( GTK_SPIN_BUTTON(widget), TRUE ); */
    gtk_spin_button_set_snap_to_ticks( GTK_SPIN_BUTTON(widget), TRUE );
    gtk_signal_connect( GTK_OBJECT(GTK_ADJUSTMENT(adj)),
                        "value_changed", GTK_SIGNAL_FUNC(adj_changed),
                        (gpointer) widget_id );
    gtk_widget_set_usize( GTK_WIDGET(&GTK_SPIN_BUTTON(widget)->entry), entry_width, 0 );
    gtk_widget_show(widget);
    gtk_box_pack_start( GTK_BOX(box), widget, FALSE, FALSE, 0 );
    gtk_widget_show(box);

    if ( widget_ptr )
        *widget_ptr = widget;

    return box;
}

/* Construct hbox of label/pixmap and scale widget, and show the widgets.
 * Returns the constructed hbox.  If widget_ptr is not null, pointer to scale
 * widget is stored in *scale_ptr.  If you pass NULL for label_text, label_width
 * will be ignored and pixmap will be created from (gchar *xpm) in place of label.
 * Pass non-zero label_width to set label to that width */
static GtkWidget *create_scale( gint min, gint max,
                                gchar *label_text, gchar **xpm,
                                gint set_val, gint page_increment,
                                GtkWidget **widget_ptr,
                                gint label_width, gint widget_id )
{
    GtkWidget *adj, *widget;
    GtkWidget *box;

    box = gtk_hbox_new( FALSE, 0 );
    /* Label */
    if ( label_text ) {
        widget = gtk_label_new(label_text);
        gtk_misc_set_alignment( GTK_MISC (widget), 0, 0 );
        gtk_widget_set_usize( widget, label_width, 0 );
    } else {
        GdkPixmap *pixmap;
        GdkBitmap *mask;

        pixmap = gdk_pixmap_create_from_xpm_d(
            main_window->window, &mask,
            &main_window->style->bg[GTK_STATE_NORMAL], xpm);
        widget = gtk_pixmap_new( pixmap, mask );
    }
    gtk_widget_show(widget);
    gtk_box_pack_start( GTK_BOX(box), widget, FALSE, FALSE, 5 );
    /* Spin button */
    adj = (GtkWidget *) gtk_adjustment_new(
        set_val, (gfloat) min, (gfloat) max,
        1.0, (gfloat) page_increment, 0.0 );
    widget = gtk_hscale_new(GTK_ADJUSTMENT(adj));
    gtk_scale_set_value_pos( GTK_SCALE(widget), GTK_POS_LEFT );
    gtk_range_set_update_policy( GTK_RANGE(widget), GTK_UPDATE_DELAYED );
    gtk_scale_set_digits( GTK_SCALE(widget), 0 );
    gtk_signal_connect( GTK_OBJECT(GTK_ADJUSTMENT(adj)),
                        "value_changed", GTK_SIGNAL_FUNC(adj_changed),
                        (gpointer) widget_id );
    gtk_box_pack_start( GTK_BOX(box), widget, TRUE, TRUE, 0 );
    gtk_widget_show(widget);
    gtk_widget_show(box);

    if ( widget_ptr )
        *widget_ptr = widget;

    return box;
}

/* Creates a hbox of controls and returns pointer to it */
GtkWidget *create_controls_box(void)
{
    GtkWidget *hbox, *vbox, *widget;
    gint label_width, entry_width;

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);

    /*
     * Ka, Ks, and Kd widgets
     */

    /* Calculate width for labels. For that, create temporary label widget, just
     * to get its font */
    widget = gtk_label_new("");
    label_width = gdk_string_width(widget->style->font, "Ka:");
    gtk_widget_destroy(widget);

    vbox = gtk_vbox_new(FALSE, 0);

    widget = create_scale( 0, num_shades - 1, "Ka:", NULL, Ka, 10,
                           &widget_Ka, label_width, KA_WIDGET );
    gtk_box_pack_start( GTK_BOX(vbox), widget, TRUE, TRUE, 0 );

    widget = create_scale( 0, num_shades - 1, "Kd:", NULL, Kd, 10,
                           &widget_Kd, label_width, KD_WIDGET );
    gtk_box_pack_start( GTK_BOX(vbox), widget, TRUE, TRUE, 0 );

    widget = create_scale( 0, num_shades - 1, "Ks:", NULL, Ks, 10,
                           &widget_Ks, label_width, KS_WIDGET );
    gtk_box_pack_start( GTK_BOX(vbox), widget, TRUE, TRUE, 0 );

    gtk_widget_show(vbox);
    gtk_box_pack_start( GTK_BOX(hbox), vbox, TRUE, TRUE, 5 );

    /* Separate groups of widgets */
    widget = gtk_vseparator_new();
    gtk_box_pack_start( GTK_BOX(hbox), widget, FALSE, FALSE, 0 );
    gtk_widget_show(widget);

    /*
     * shades widget
     */

    /* Calculate width for entry (inside shade spin button). For that, create
     * temporary entry widget, just to get its font (we actually add more digits
     * than needed to allow for the two `spin triangles' and hope this will give
     * correct width) */
    widget = gtk_entry_new();
    entry_width = gdk_string_width(widget->style->font, "000000");
    gtk_widget_destroy(widget);

    /* Note that widget_shades must be created after widget_K*, because widget's
     * code will refer to widget_K*'s */
    vbox = gtk_vbox_new(FALSE, 0);

    widget = create_spin_button( 2, 256, "Shades", num_shades, 10, &widget_shades,
                                 0, entry_width, SHADES_WIDGET );
    gtk_box_pack_start( GTK_BOX(vbox), widget, TRUE, FALSE, 0 );
    gtk_widget_show(vbox);
    gtk_box_pack_start( GTK_BOX(hbox), vbox, FALSE, FALSE, 5 );

    /* Separate groups of widgets */
    widget = gtk_vseparator_new();
    gtk_box_pack_start( GTK_BOX(hbox), widget, FALSE, FALSE, 0 );
    gtk_widget_show(widget);

    /*
     * r, g, and b widgets
     */

    /* Calculate width for labels. For that, create temporary label widget, just
     * to get its font */
    widget = gtk_label_new("");
    label_width = gdk_string_width(widget->style->font, "G:");
    gtk_widget_destroy(widget);

    vbox = gtk_vbox_new(FALSE, 5);

    widget = create_scale( 0, 100, NULL, (gchar **) red_xpm, red, 10, &widget_red,
                           label_width, RED_WIDGET);
    gtk_box_pack_start( GTK_BOX(vbox), widget, TRUE, TRUE, 0 );

    widget = create_scale( 0, 100, NULL, (gchar **) green_xpm, green, 10, &widget_green,
                           label_width, GREEN_WIDGET );
    gtk_box_pack_start( GTK_BOX(vbox), widget, TRUE, TRUE, 0 );

    widget = create_scale( 0, 100, NULL, (gchar **) blue_xpm, blue, 10, &widget_blue,
                           label_width, BLUE_WIDGET );
    gtk_box_pack_start( GTK_BOX(vbox), widget, TRUE, TRUE, 0 );

    gtk_widget_show(vbox);
    gtk_box_pack_start( GTK_BOX(hbox), vbox, TRUE, TRUE, 5 );


    gtk_widget_show(hbox);

    /* Prepare the palette */
    if ( !setup_palette() )
        return NULL;

    return hbox;
}



/*
 * Local variables:
 * compile-command: "make -k && ./ball"
 * End:
 */
