#include <fcntl.h>
#include <stdlib.h>

#include "wrapper.h"

#ifdef __LINUX__
#include <sys/stat.h>

#else /* __LINUX__ */
#include <sys\stat.h>
#include <io.h>

#endif /* __LINUX__ */

static int fontwidth;
static int fontheight;
static unsigned char *font = NULL;

int loadfont(int width, int height, const char *path)
{
  int fontfile, count;

  if ( width <= 0 || height <= 0 )
    return 0;

  if ( font ) {
    free( (void *) font );
    font = NULL;
  }

  fontwidth = width; /* pixel width */
  fontheight = height; /* pixel height */

  width = (width + 7) / 8; /* number of bytes per scanline */
#ifdef __LINUX__
  fontfile = open(path, O_RDONLY);
#else /* __LINUX__ */
  fontfile = open(path, O_RDONLY | O_BINARY);
#endif /* __LINUX__ */
  if ( fontfile == -1 )
    return 0;

  count = width*height*256;
  font = (unsigned char *) malloc(count);
  if ( !font ) {
    close(fontfile);
    return 0;
  }

  if ( count != read(fontfile, (void *) font, count) ) {
    close(fontfile);
    font = NULL;
    return 0;
  }

  close(fontfile);
  return 1;
}

void textout(int x, int y, const unsigned char *str)
{
  int i, j, k, scrx, bytes_per_scanline;
  unsigned char *fontptr, mask;
  int bytes_per_charcell;

  bytes_per_scanline = (fontwidth + 7) / 8;
  bytes_per_charcell =
    bytes_per_scanline * fontheight;

  while ( *str ) {
    fontptr =
      font + bytes_per_charcell * *str; /* start of image in the buffer */
    for ( j = 0; j < fontheight; j++ ) {/* draw every scanline */
      scrx = x;                         /* every scanline starts at x */
      for ( i = 0; i < bytes_per_scanline; i++ ) {
        mask = 0x80;
        for ( k = 0; k < 8; k++) { /* !!! Bug---does not work when fontwidth % 8 != 0 !!!*/
          if ( mask & *(fontptr + i) )
            vga_drawpixel(scrx + k, y + j);
          mask >>= 1;
        }
        scrx += 8;
      }
      fontptr += bytes_per_scanline;
    }
    str++;
    x += fontwidth;
  }
}
