#include "list.h"


list::~list(void)
{
    litem *curr;

    /* Delete any remaining litem's */
    while ( _head ) {
        curr = _head;
        _head = _head->next;
        delete curr;
    }

    return;
}

litem *list::add(litem *item)
{
    if ( _head ) { /* list is not empty */
        assert( _tail );
        assert( _count );
        _tail->next = item;
        _tail = item;
    } else {       /* list is empty */
        assert ( !_tail );
        assert ( !_count );
        _head = _tail = item;
    }

    _tail->next = NULL;
    _count++;

    return _tail;
}

list *list::copy(void)
{
    list  *nlist = new list;
    litem *curr;

    curr = _head;

    while ( curr ) {
        nlist->add(curr->copy());
        curr = curr->next;
    }

    return nlist;
}

litem *list::next_cycled(litem *curr)
{
    if ( curr )
        if ( curr->next )
            return curr->next;
        else
            return _head;

    return NULL;
}

int list::contains(litem *item)
{
    litem *curr;

    if ( !item )
        return 0;

    curr = _head;
    while ( curr ) { /* search for item */
        if ( curr == item )
            break;
        curr = curr->next;
    }

    return (int) (curr == item);
}

int list::del(litem *item)
{
    litem *curr, *prev;

    curr = _head;
    prev = NULL;
    while ( curr ) { /* search for item */
        if ( curr == item )
            break;
        prev = curr;
        curr = curr->next;
    }

    if ( curr ) {    /* found */
        if ( prev )  /* not head */
            prev->next = curr->next;
        else         /* head */
            _head = curr->next;
        if ( !curr->next ) /* tail */
            _tail = prev;

        delete curr;/* finish it */
        _count--;
    }

    return (int) curr;
}

void list::draw(GdkDrawable *drawable, GdkGC *gc)
{
    litem *curr = _head;

    while ( curr ) {
        curr->draw( drawable, gc );
        curr = curr->next;
    }

    return;
}

void list::print(FILE *stream)
{
    litem *curr = _head;

    while ( curr ) {
        curr->print(stream);
        curr = curr->next;
    }

    return;
}


/*
 * Local variables:
 * compile-command: "make -k && ./triangulate < polygon.pts"
 * End:
 */
