#include <assert.h>

#define SCREEN_WIDTH        320

static unsigned char *scr;
static int            scr_width;
static unsigned char  col;

static void diagonal(int x, int y, int deltaY, int dirX);
static void horizontal(int x, int y, int deltaX, int dirX);
static void vertical(int x, int y, int deltaY);
static void octant0(int x, int y, int deltaX, int deltaY, int dirX);
static void octant1(int x, int y, int deltaX, int deltaY, int dirX);


#define draw_pixel(x,y)     ( *(scr + y*scr_width + x) = col )

void line(unsigned char *screen, unsigned char color,
          int x, int y, int X, int Y)
{
    int tmp, deltaX, deltaY, dirX;

    assert(screen);

    scr       = screen;
    scr_width = SCREEN_WIDTH;
    col       = color;

    if ( y > Y ) { /* all functions assume y <= Y */
        tmp = y, y = Y, Y = tmp;
        tmp = x, x = X, X = tmp;
    }

    deltaX = X - x;
    if ( deltaX < 0 ) {
        deltaX = -deltaX; /* | deltaX | */
        dirX = -1;
    } else
        dirX = 1;
    deltaY = Y - y;       /* deltaY >= 0 */

    if ( deltaX == 0 )           /* vertical */
        vertical(x, y, deltaY);
    else if ( deltaY == 0 )      /* horizontal */
        horizontal(x, y, deltaX, dirX);
    else if ( deltaX == deltaY ) /* diagonal */
        diagonal(x, y, deltaY, dirX);
    else if ( deltaX < deltaY )  /* octant 1 or 2 ( deltaX != 0 ) */
        octant1(x, y, deltaX, deltaY, dirX);
    else                         /* octant 0 or 3 ( deltaX > deltaY != 0 ) */
        octant0(x, y, deltaX, deltaY, dirX);

/*     if ( deltaX > deltaY ) */
/*         octant0(x, y, deltaX, deltaY, dirX); */
/*     else */
/*         octant1(x, y, deltaX, deltaY, dirX); */

    return;
}

/*
 * Draws horizontal lines
 */
static void horizontal(int x, int y, int deltaX, int dirX)
{
    assert( deltaX >= 0 );

    while ( deltaX-- >= 0 ) {
        draw_pixel(x, y);
        x += dirX;
    }

    return;
}

/*
 * Draws vertical lines
 */
static void vertical(int x, int y, int deltaY)
{
    assert( deltaY >= 0 );

    while ( deltaY-- >= 0 )
        draw_pixel(x, y++);

    return;
}

/*
 * Draws diagonal (45 or 135 degrees) and vertical lines
 */
static void diagonal(int x, int y, int deltaY, int dirX)
{
    assert( deltaY >= 0 );

    while ( deltaY-- >= 0 ) {
        draw_pixel(x, y++);
        x += dirX;
    }

    return;
}

/*
 * Draws lines in octants 0 and 3 ( |deltaX| > deltaY )
 */
static void octant0(int x, int y, int deltaX, int deltaY, int dirX)
{
    int error, errorAdd, errorReset, thresh;

    assert( deltaY >= 0 );

    /* Scaled error threshold for moving up one pixel */
    thresh = deltaX;
    errorReset = 2 * thresh;
    /* Scaled y-increment corresponding to x-increment of 1 */
    errorAdd = 2 * deltaY;

    /* Initial error */
    error  = 0;

    while ( deltaX-- >= 0) {
        draw_pixel(x, y);
        /* Step 1 pixel sideways, accumulating error in direction of Oy */
        x += dirX;
        error += errorAdd;
        if ( error >= thresh ) { /* threshold reached, go up */
            y++;
            error -= errorReset;  /* adjust back the error */
        }
    }

    return;
}

/*
 * Draws lines in octants 1 and 2 ( |deltaX| < deltaY )
 */
static void octant1(int x, int y, int deltaX, int deltaY, int dirX)
{
    int error, errorAdd, errorReset, thresh;

    assert( deltaY >= 0 );

    /* Scaled error threshold for moving sideways one pixel */
    thresh = deltaY;
    errorReset = 2 * thresh;
    /* Scaled x-increment corresponding to y-increment of 1 */
    errorAdd = 2 * deltaX;

    /* Initial error */
    error  = 0;

    while ( deltaY-- >= 0 ) {
        draw_pixel(x, y);
        /* Step 1 pixel up Oy, accumulating error in direction of Ox */
        y++;
        error += errorAdd;
        if ( error >= thresh ) { /* threshold reached, go sideways */
            x += dirX;
            error -= errorReset; /* adjust back the error */
        }
    }

    return;
}



/*
 * Local variables:
 * compile-command: "make -k"
 * End:
 */
