#include <assert.h>

#include "wrapper.h"

/*
 * !!!These draw unbalanced segments, so don't use it!!!
 */


#define SCREEN_WIDTH        320

static unsigned char *scr;
static int            scr_width;
static unsigned char  col;

static void diagonal(int x, int y, int deltaY, int dirX);
static void horizontal(int x, int y, int deltaX, int dirX);
static void vertical(int x, int y, int deltaY);
static void octant0(int x, int y, int deltaX, int deltaY, int dirX);
static void octant1(int x, int y, int deltaX, int deltaY, int dirX);

#define draw_pixel(x,y)     ( *(scr + (y)*scr_width + x) = col )
static void draw_horiz_run(int x, int y, int len, int dirX);
static void draw_vert_run(int x, int y, int len);



void line_segments(unsigned char *screen, unsigned char color,
                   int x, int y, int X, int Y)
{
    int tmp, deltaX, deltaY, dirX;

    assert(screen);

    scr       = screen;
    scr_width = SCREEN_WIDTH;
    col       = color;

    if ( y > Y ) { /* all functions assume y <= Y */
        tmp = y, y = Y, Y = tmp;
        tmp = x, x = X, X = tmp;
    }

    deltaX = X - x;
    if ( deltaX < 0 ) {
        deltaX = -deltaX; /* | deltaX | */
        dirX = -1;
    } else
        dirX = 1;
    deltaY = Y - y;       /* deltaY >= 0 */

    if ( deltaX == 0 )           /* vertical */
        vertical(x, y, deltaY);
    else if ( deltaY == 0 )      /* horizontal */
        horizontal(x, y, deltaX, dirX);
    else if ( deltaX == deltaY ) /* diagonal */
        diagonal(x, y, deltaY, dirX);
    else if ( deltaX < deltaY )  /* octant 1 or 2 ( deltaX != 0 ) */
        octant1(x, y, deltaX, deltaY, dirX);
    else                         /* octant 0 or 3 ( deltaX > deltaY != 0 ) */
        octant0(x, y, deltaX, deltaY, dirX);

/*     if ( deltaX > deltaY ) */
/*         octant0(x, y, deltaX, deltaY, dirX); */
/*     else */
/*         octant1(x, y, deltaX, deltaY, dirX); */

    return;
}

/*
 * Draws horizontal lines
 */
static void horizontal(int x, int y, int deltaX, int dirX)
{
    assert( deltaX >= 0 );

    while ( deltaX-- >= 0 ) {
        draw_pixel(x, y);
        x += dirX;
    }

    return;
}

/*
 * Draws vertical lines
 */
static void vertical(int x, int y, int deltaY)
{
    assert( deltaY >= 0 );

    while ( deltaY-- >= 0 )
        draw_pixel(x, y++);

    return;
}

/*
 * Draws diagonal (45 or 135 degrees) and vertical lines
 */
static void diagonal(int x, int y, int deltaY, int dirX)
{
    assert( deltaY >= 0 );

    while ( deltaY-- >= 0 ) {
        draw_pixel(x, y++);
        x += dirX;
    }

    return;
}

/*
 * Draws lines in octants 0 and 3 ( |deltaX| > deltaY )
 */
static void octant0(int x, int y, int deltaX, int deltaY, int dirX)
{
    int error, errorAdd, errorReset, thresh, seglen, growX;

    assert( deltaY >= 0 );
    deltaX++, deltaY++;

    seglen = deltaX / deltaY;
    growX  = dirX < 0 ? -seglen : seglen;

    /* Scaled error threshold for adding one pixel to seglen */
    thresh = deltaY;
    errorReset = 2 * thresh;
    /* Scaled error increment in direction of Ox when moving up 1 pixel */
    errorAdd = 2 * (deltaX % deltaY);

    /* Initial error */
    error = 0;

    while ( deltaY-- > 0 ) {
        error += errorAdd;
        if ( error >= thresh ) { /* threshold reached, go up */
            draw_horiz_run(x, y, seglen + 1, dirX);
            error -= errorReset; /* adjust back the error */
            x += dirX;
        } else
            draw_horiz_run(x, y, seglen, dirX);
        y++;
        x += growX;
    }

    return;
}

static void octant1(int x, int y, int deltaX, int deltaY, int dirX)
{
    int error, errorAdd, errorReset, thresh, seglen;

    assert( deltaY >= 0 );
    deltaX++, deltaY++;

    seglen = deltaY / deltaX;

    /* Scaled error threshold for adding one pixel to seglen */
    thresh = deltaX;
    errorReset = 2 * thresh;
    /* Scaled error increment in direction of Oy when moving sideways 1 pixel */
    errorAdd = 2 * (deltaY % deltaX);

    /* Initial error */
    error  = 0;

    while ( deltaX-- > 0 ) {
        error += errorAdd;
        if ( error >= thresh ) { /* threshold reached, go up */
            draw_vert_run(x, y++, seglen + 1);
            error -= errorReset; /* adjust back the error */
        } else
            draw_vert_run(x, y, seglen);
        y += seglen;
        x += dirX;
    }

    return;
}

static void draw_horiz_run(int x, int y, int len, int dirX)
{
    while ( len-- ) {
        draw_pixel(x, y);
        x += dirX;
    }

    return;
}

static void draw_vert_run(int x, int y, int len)
{
    while ( len-- ) {
        draw_pixel(x, y++);
    }

    return;
}

/*
 * Local variables:
 * compile-command: "make -k"
 * End:
 */
