#! /usr/bin/python

import sys
import getopt
from math import *

# Default values
color    = 2
step     = .1
cont     = 0
xmin     = -1
xmax     = 1
ymin     = -1
ymax     = 1
label    = ""
uselabel = 0

# Error messages
errTryHelp          = "Try `" + sys.argv[0] + " -h' for help\n"
errFloatingPointReq = "Option `%s' requires a floating-point argument or a script evaluating to such\n"
errIntegerReq       = "Option `%s' requires an integer argument or a script evaluating to such\n"
errScript           = "You must specify exactly one script as a function of variable `x'\n"

usage = \
"Usage: " + sys.argv[0] + """ [-h] [-x xmin] [-y ymin] [-X xmax] [-Y ymax]
                [-s step] [-c color] [-l label] [-C] script

-h        show this help message
-x xmin   x-coord of ll corner of real-world viewport (default: """ + str(xmin) + """)
-y ymin   y-coord of ll corner of real-world viewport (default: """ + str(ymin) + """)
-X xmax   x-coord of ur corner of real-world viewport (default: """ + str(xmax) + """)
-Y ymax   y-coord of ur corner of real-world viewport (default: """ + str(ymax) + """)
-s step   step for x argument (default: """ + str(step) + """)
-c color  integer color to use for graph (default: """ + str(color) + """)
-l label  use specified label instead of function script
-C        treat the first point as a "draw-to" point (default: the
          first point is treated as a "move-to" point)
script    python script which should be a function of variable `x'
          evaluating to floating-point number

Note: coordinates and step take arguments which must be floating-point numbers or
      expressions evaluating to such
"""

# Process command line arguments
try:
    optionlist, args = getopt.getopt(sys.argv[1:], "x:X:y:Y:s:c:l:Ch")
except getopt.GetoptError, (msg, opt):
    sys.stderr.write(msg + "\n")
    sys.stderr.write(errTryHelp)
    sys.exit(1)

for opt, val in optionlist:
    try:
        if opt == "-x":
            xmin = float(eval(val))
        if opt == "-X":
            xmax = float(eval(val))
        if opt == "-y":
            ymin = float(eval(val))
        if opt == "-Y":
            ymax = float(eval(val))
        if opt == "-s":
            step = float(eval(val))
    except (ValueError, SyntaxError, NameError):
        sys.stderr.write(errFloatingPointReq % (opt))
        sys.stderr.write(errTryHelp)
        sys.exit(1)

    try:
        if opt == "-c":
            color = int(eval(val))
    except (ValueError, SyntaxError, NameError):
        sys.stderr.write(errIntegerReq % (opt))
        sys.stderr.write(errTryHelp)
        sys.exit(1)

    if opt == "-l":
        label = val
        uselabel = 1
    if opt == "-C":
        cont = 1
    if opt == "-h":
        print usage
        sys.exit(0)

# Exactly one argument (function script) must be left
if len(args) != 1:
    sys.stderr.write(errScript)
    sys.stderr.write(errTryHelp)
    sys.exit(1)

# Output function script or label
if uselabel:
    print label
else:
    print args[0]

# Output color and real-world viewport coordinates
print color, xmin, ymin, xmax, ymax

# Output coordinate pairs
x = xmin
print cont, x, eval(args[0])
x += step
while x <= xmax:
    print 1, x, eval(args[0])
    x += step
