#! /usr/bin/python

import sys
import getopt
from math import *

#
#  Quadrants:
#          ORIENT == 1                ORIENT == 0
#       -----------------          -----------------
#      |     |     |     |        |     |     |     |
#      |x2y2 |x3y3 |x8y8 |        |x8y8 |x3y3 |x2y2 |
#      |-----|-----|-----|        |-----|-----|-----|
#      |     |     |     |        |     |     |     |
#      |x1y1 |x4y4 |x7y7 |        |x7y7 |x4y4 |x1y1 |
#      |-----|-----|-----|        |-----|-----|-----|
#      |     |     |     |        |     |     |     |
#      |x0y0 |x5y5 |x6y6 |        |x6y6 |x5y5 |x0y0 |
#  (X,Y)-----------------     (X,Y)-----------------
#
#
#  Directions:     Orientation:          O | D | Starting Quadrant
#      0                                ---|---|-------------------
#      |           1 - downside down     1 | 0 | 8
#  - <- -> -                             1 | 1 | 0
#      |           0 - upside down       0 | 0 | 8
#      1                                 0 | 1 | 0
#
def peano(X, Y, A, DIR, ORIENT):
    a          = A / 3
    aa         = a + a
    # Coordinates of quadrants
    x, y       = [0,0,0, 0,0,0, 0,0,0], [0,0,0, 0,0,0, 0,0,0]
    x[3], y[3] = X + a,  Y + aa
    x[4], y[4] = X + a,  Y + a
    x[5], y[5] = X + a,  Y
    if ORIENT == 1:   # downside down
        x[0], y[0] = X,      Y
        x[1], y[1] = X,      Y + a
        x[2], y[2] = X,      Y + aa
        x[6], y[6] = X + aa, Y
        x[7], y[7] = X + aa, Y + a
        x[8], y[8] = X + aa, Y + aa
    else:             # upside down
        x[0], y[0] = X + aa, Y
        x[1], y[1] = X + aa, Y + a
        x[2], y[2] = X + aa, Y + aa
        x[6], y[6] = X,      Y
        x[7], y[7] = X,      Y + a
        x[8], y[8] = X,      Y + aa
    # Index within norder of starting quadrant and increment value
    if DIR == 1:
        quadrant = 8
        q = -1
    else:
        quadrant = 0
        q = 1

    if a == 1:
        for i in [2, 1, 2, 1, 2, 0]: # draw the wiggle inside 3x3 square
            print 1, x[quadrant], y[quadrant]
            quadrant += q * i
    else:
        for i in range(3): # recursively draw all nine quadrants in order
            for j in range(3):
                peano(x[quadrant], y[quadrant], a, DIR, ORIENT)
                quadrant += q
                ORIENT = not ORIENT
            DIR = not DIR



# Default values
color    = 7
side     = 3
label    = "Peano Curve (order %d)"
use_default_label = 1

# Error messages
errTryHelp          = "Try `" + sys.argv[0] + " -h' for help\n"
errIntegerReq       = "Option `%s' requires an integer argument or a script evaluating to such\n"
errSideRange        = "Square side parameter must be a positive integer\n"


usage = \
"Usage: " + sys.argv[0] + """ [-h] [-s side] [-c color] [-l label] [-C]

-h        show this help message
-s side   integer parameter for square side (square side == 3**side)
-c color  integer color to use for graph (default: """ + str(color) + """)
-l label  use specified label (default: `""" + label + """')
"""



# Process command line arguments
try:
    optionlist, args = getopt.getopt(sys.argv[1:], "s:c:l:h")
except getopt.GetoptError, (msg, opt):
    sys.stderr.write(msg + "\n")
    sys.stderr.write(errTryHelp)
    sys.exit(1)

for opt, val in optionlist:
    try:
        if opt == "-c":
            color = int(eval(val))
        if opt == "-s":
            side = int(val)
            if side <= 0:
                sys.stderr.write(errSideRange)
                sys.stderr.write(errTryHelp)
                sys.exit(1)
    except (ValueError, SyntaxError, NameError):
        sys.stderr.write(errIntegerReq % (opt))
        sys.stderr.write(errTryHelp)
        sys.exit(1)

    if opt == "-l":
        label = val
        use_default_label = 0
    if opt == "-h":
        print usage
        sys.exit(0)

if use_default_label:
    label = label % side

# Output label
print label

# Output color and real-world viewport coordinates
print color, 0, 0, 3**side, 3**side

print 0, 0, 0 # "move-to"
peano(0, 0, 3**side, 0, 1)
