#! /usr/bin/python

import sys
import getopt
from math import *

#
# (x5, y5)_________ (x3, y3)
#        |        /\
#      b |      /    \
#        |    /        \
#      alpha/            \
#        |/                \
# (x4, y4)\                / (x2, y2)
#           \        a   /
#             \        /
#               \    /
#                 \/ beta
#      ------------------------
#              (x1, y1)
#

def pythagor(x1, y1, x2, y2, depth):
    if depth <= 0:
        return

    a                   = sqrt( (x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1) )
    sin_beta            = (y2 - y1) / a
    cos_beta            = (x2 - x1) / a
    cos_alpha_plus_beta = cos_alpha * cos_beta - sin_alpha * sin_beta
    sin_alpha_plus_beta = sin_alpha * cos_beta + cos_alpha * sin_beta
    b                   = a * cos_alpha

    x3, y3 = x2 - a * sin_beta,            y2 + a * cos_beta
    x4, y4 = x1 - a * sin_beta,            y1 + a * cos_beta
    x5, y5 = x4 + b * cos_alpha_plus_beta, y4 + b * sin_alpha_plus_beta

    print 0, x4, y4 # move to
    print 1, x3, y3
    print 1, x2, y2
    print 1, x1, y1
    print 1, x4, y4
    print 1, x5, y5
    print 1, x3, y3
    pythagor(x4, y4, x5, y5, depth - 1)
    pythagor(x5, y5, x3, y3, depth - 1)

def pythagor_detect(x1, y1, x2, y2, depth):
    global xmin_det, ymin_det, xmax_det, ymax_det

    if depth <= 0:
        return

    a                   = sqrt( (x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1) )
    sin_beta            = (y2 - y1) / a
    cos_beta            = (x2 - x1) / a
    cos_alpha_plus_beta = cos_alpha * cos_beta - sin_alpha * sin_beta
    sin_alpha_plus_beta = sin_alpha * cos_beta + cos_alpha * sin_beta
    b                   = a * cos_alpha

    x3, y3 = x2 - a * sin_beta,            y2 + a * cos_beta
    x4, y4 = x1 - a * sin_beta,            y1 + a * cos_beta
    x5, y5 = x4 + b * cos_alpha_plus_beta, y4 + b * sin_alpha_plus_beta

    xmax_det = max(x1, x2, x3, x4, x5, xmax_det)
    ymax_det = max(y1, y2, y3, y4, y5, ymax_det)
    xmin_det = min(x1, x2, x3, x4, x5, xmin_det)
    ymin_det = min(y1, y2, y3, y4, y5, ymin_det)

    pythagor_detect(x4, y4, x5, y5, depth - 1)
    pythagor_detect(x5, y5, x3, y3, depth - 1)



# Default values
xmin     = 0
det_xmin = 1
xmax     = 0
det_xmax = 1
ymin     = 0
det_ymin = 1
ymax     = 0
det_ymax = 1
u        = 0
v        = 0
side     = 10
alpha_raw= "pi/4"
alpha    = eval(alpha_raw)
depth    = 8
color    = 7
label    = "Pythagorean Tree (order %d)"
use_default_label = 1

# Error messages
errTryHelp          = "Try `" + sys.argv[0] + " -h' for help\n"
errFloatingPointReq = "Option `%s' requires a floating-point argument or a script evaluating to such\n"
errIntegerReq       = "Option `%s' requires an integer argument or a script evaluating to such\n"
errAngleRange       = "Angle must be a floating-point number from range (0, pi/2) or a script\n\
evaluating to such\n"
errSideRange        = "Square side must be a positive floating-point number or a script\n\
evaluating to such\n"
errDepthRange       = "Recursion depth must be a positive integer number or a script\n\
evaluating to such\n"

usage = \
"Usage: " + sys.argv[0] + """ [-h] [-x xmin] [-y ymin] [-X xmax] [-Y ymax]
                [-u x0] [-v y0] [-s side] [-a angle] [-d depth] [-c color]
                [-l label]

-h        show this help message
-x xmin   x-coord of ll corner of real-world viewport (default: autodetect)
-y ymin   y-coord of ll corner of real-world viewport (default: autodetect)
-X xmax   x-coord of ur corner of real-world viewport (default: autodetect)
-Y ymax   y-coord of ur corner of real-world viewport (default: autodetect)
-u x0     x-coord of ll corner of first-order square (default: """ + str(u) + """)
-v y0     y-coord of ll corner of first-order square (default: """ + str(v) + """)
-s side   length of the side of first-order square (default: """ + str(side) + """)
-a angle  acute angle of right-angled triangle (default: `""" + str(alpha_raw) + """')
-d depth  recursion depth (default: """ + str(depth) + """)
-c color  integer color to use for drawing (default: """ + str(color) + """)
-l label  use specified label (default: `""" + label + """')

Note: coordinates, angle and square side length take arguments which must be
      floating-point numbers or expressions evaluating to such;
      angle is taken to be in radians
"""



# Process command line arguments
try:
    optionlist, args = getopt.getopt(sys.argv[1:], "x:y:X:Y:u:v:s:a:d:c:l:h")
except getopt.GetoptError, (msg, opt):
    sys.stderr.write(msg + "\n")
    sys.stderr.write(errTryHelp)
    sys.exit(1)

for opt, val in optionlist:
    try:
        if opt == "-x":
            xmin = float(eval(val))
            det_xmin = 0
        if opt == "-X":
            xmax = float(eval(val))
            det_xmax = 0
        if opt == "-y":
            ymin = float(eval(val))
            det_ymin = 0
        if opt == "-Y":
            ymax = float(eval(val))
            det_ymax = 0
        if opt == "-u":
            u = float(eval(val))
        if opt == "-v":
            v = float(eval(val))
        if opt == "-s":
            side = float(eval(val))
            if side <= 0:
                sys.stderr.write(errSideRange)
                sys.stderr.write(errTryHelp)
                sys.exit(1)
        if opt == "-a":
            alpha = float(eval(val))
            if alpha <= 0 and alpha >= pi/2:
                sys.stderr.write(errAngleRange)
                sys.stderr.write(errTryHelp)
                sys.exit(1)
    except (ValueError, SyntaxError, NameError):
        sys.stderr.write(errFloatingPointReq % (opt))
        sys.stderr.write(errTryHelp)
        sys.exit(1)

    try:
        if opt == "-c":
            color = int(eval(val))
        if opt == "-d":
            depth = int(eval(val))
            if depth <= 0:
                sys.stderr.write(errDepthRange)
                sys.stderr.write(errTryHelp)
                sys.exit(1)
    except (ValueError, SyntaxError, NameError):
        sys.stderr.write(errIntegerReq % (opt))
        sys.stderr.write(errTryHelp)
        sys.exit(1)

    if opt == "-l":
        label = val
        use_default_label = 0
    if opt == "-h":
        print usage
        sys.exit(0)

# Calculate coordinate ranges
sin_alpha = sin(alpha)
cos_alpha = cos(alpha)

if det_xmin or det_ymin or det_xmax or det_ymax:
    xmin_det =  1.7976931348623157e+308 # hope it's right
    xmax_det = -1.7976931348623157e+308 # hope it's right
    ymin_det =  1.7976931348623157e+308 # hope it's right
    ymax_det = -1.7976931348623157e+308 # hope it's right
    pythagor_detect(u, v, u + side, v, depth)
    if det_xmin: xmin = xmin_det
    if det_ymin: ymin = ymin_det
    if det_xmax: xmax = xmax_det
    if det_ymax: ymax = ymax_det

if use_default_label:
    label = label % depth

# Output label
print label

# Output color and real-world viewport coordinates
print color, xmin, ymin, xmax, ymax

# Kick off the recursion
pythagor(u, v, u + side, v, depth)
