#include "splines.h"


/* Creates empty points list (except for two invisible start and end points) */
pnode *points_init(void)
{
    pnode *points;

    /* Create two `invisible' points */
    points       = (pnode *) malloc( sizeof(pnode) );
    if ( !points ) return NULL;
    points->next = (pnode *) malloc( sizeof(pnode) );
    if ( !points->next ) return NULL;
    points->next->next = NULL;

    return points;
}

void point_remove(pnode *haystack, pnode *needle)
{
    pnode *prev, *curr;

    /* Sanity checks */
    if ( !haystack ) /* no haystack */
        return;
    if ( !needle )   /* no needle */
        return;
    assert(haystack->next); /* haystack should always contain at least two
                             * `invisible' points */
    if ( !haystack->next->next ) /* no visible points in the haystack */
        return;

    curr = haystack;
    while ( curr ) {
        prev = curr;
        curr = curr->next;
        if ( curr == needle ) {      /* found */
            prev->next = curr->next; /* unlink */
            free( (void *) curr );   /* free */
            if ( prev == haystack )
                /* This was first point, so it had invisible preceding partner */
                prev->p = prev->next->p;
            if ( !prev->next->next )
                /* This is last point, so it has invisible trailing partner */
                prev->next->p = prev->p;
            break;
        }
    }

    return;
}

/* Returns 1 if error (NULL list, point or memory allocation failed); 0
 * otherwise */
pnode *point_add(pnode *haystack, GdkPoint *point)
{
    pnode *curr;

    /* Sanity checks */
    if ( !haystack ) /* nowhere to add to */
        return NULL;
    if ( !point )    /* nothing to add */
        return NULL;
    assert(haystack->next); /* haystack should always contain at least two
                             * `invisible' points */

    curr = haystack;
    while ( curr->next ) /* skip to the end */
        curr = curr->next;

    if ( curr == haystack->next ) { /* list was empty before (except for the */
        haystack->p = *point;       /* two invisible points) */
        haystack->next = curr;
    }
    curr->p = *point;
    curr->next = (pnode *) malloc( sizeof(pnode) );
    if ( !curr->next )
        return NULL;
    /* Trailing invisible point */
    curr->next->p = *point;
    curr->next->next = NULL;

    return curr;
}

/* Returns (last) point from the list in whose proximity is coord pixel */
pnode *point_from_coords(pnode *points, GdkPoint *coord, gint16 point_radius)
{
    pnode *p = NULL;

    if ( points ) {
        /* Skip first `invisible' point */
        points = points->next;
        while ( points && points->next ) { /* skip last `invisible' point */
            if ( coord->x >= points->p.x - point_radius &&
                 coord->x <= points->p.x + point_radius &&
                 coord->y >= points->p.y - point_radius &&
                 coord->y <= points->p.y + point_radius    )
                p = points;
            points = points->next;
        }
    }
    return p;
}

void draw_points(GdkDrawable *drawable, GdkGC *gc, pnode *points, gint radius)
{
    pnode *curr, *next;

    if ( !points ||
         !(curr = points->next) ) /* skip first `invisible' point */
        return;

    next = curr->next;

    while ( next ) { /* skip last `invisible' point */
        gdk_draw_arc( drawable, gc, FALSE,
                      curr->p.x-radius, curr->p.y-radius,
                      radius+radius, radius+radius, 0, 360*64 );
        curr = next;
        next = curr->next;
    }

    return;
}



/*
 * Local variables:
 * compile-command: "make -k && ./splines"
 * End:
 */
