#include "splines.h"



/* The first and last segments are not drawn--the first and last point are in
 * the list only for purposes of calculation of other segments.  Note that you
 * can make two copies of the first point at the head of the list and two copies
 * of the last point at the tail if you want the segments to be drawn.  Beware
 * that you can overflow integers if you give too big `gran' (the maximum
 * depends on your maximum x and y values) */
void draw_splines(GdkDrawable *drawable, GdkGC *gc, pnode *points, unsigned gran)
{
    int      t, x, y, xlast, ylast;
    int      a0, a1, a2, a3, b0, b1, b2, b3;
    pnode   *z_im1, *z_i, *z_ip1, *z_ip2;
    long     gran3, gran_sq3, gran_cube;

    if ( !(z_im1 = points)      ||
         !(z_i   = z_im1->next) ||
         !(z_ip1 = z_i->next)   ||
         !(z_ip2 = z_ip1->next)    )
        return;

    gran3     = gran * 3;
    gran_sq3  = gran3 * gran;
    gran_cube = gran * gran * gran;

    a0 =  z_im1->p.x + 4*z_i->p.x +   z_ip1->p.x;
    b0 =  z_im1->p.y + 4*z_i->p.y +   z_ip1->p.y;
    xlast = a0/6; /* t == 0 */
    ylast = b0/6; /* t == 0 */

    do {
        /* The formulas are:
         *
         * $$ \eqalign{
         *    x(t) = a_3t^3 + a_2t^2 + a_1t + a_0,
         *    y(t) = b_3t^3 + b_2t^2 + b_1t + b_0,
         * } $$
         *
         * where
         *
         * $$ \eqalign{
         *    a_3 &= { -x_{i-1} + 3x_i - 3x_{i+1} + x_{i+2} \over 6 },\cr
         *    a_2 &= { x_{i-1} - 2x_i + x_{i+1} \over 2 },\cr
         *    a_1 &= { -x_{i-1} + x_{i+1} \over 2 },\cr
         *    a_0 &= { x_{i-1} + 4x_i + x_{i+1} \over 6 },\cr
         *    \cr
         *    b_3 &= { -y_{i-1} + 3y_i - 3y_{i+1} + y_{i+2} \over 6 },\cr
         *    b_2 &= { y_{i-1} - 2y_i + y_{i+1} \over 2 },\cr
         *    b_1 &= { -y_{i-1} + y_{i+1} \over 2 },\cr
         *    b_0 &= { y_{i-1} + 4y_i + y_{i+1} \over 6 }.
         * } $$
         */
        a0 =  z_im1->p.x + 4*z_i->p.x +   z_ip1->p.x;
        a1 = -z_im1->p.x             +    z_ip1->p.x;
        a2 =  z_im1->p.x - 2*z_i->p.x +   z_ip1->p.x;
        a3 = -z_im1->p.x + 3*z_i->p.x - 3*z_ip1->p.x + z_ip2->p.x;

        b0 =  z_im1->p.y + 4*z_i->p.y +   z_ip1->p.y;
        b1 = -z_im1->p.y              +   z_ip1->p.y;
        b2 =  z_im1->p.y - 2*z_i->p.y +   z_ip1->p.y;
        b3 = -z_im1->p.y + 3*z_i->p.y - 3*z_ip1->p.y + z_ip2->p.y;

        for ( t = 0; t <= gran; t++ ) {
            x = (((a3*t + gran3*a2)*t + gran_sq3*a1)*t + gran_cube*a0) / (6*gran_cube);
            y = (((b3*t + gran3*b2)*t + gran_sq3*b1)*t + gran_cube*b0) / (6*gran_cube);

            gdk_draw_line( drawable, gc, xlast, ylast, x, y );
            xlast = x;
            ylast = y;

        }
        z_im1 = z_i;
        z_i   = z_ip1;
        z_ip1 = z_ip2;
        z_ip2 = z_ip2->next;
    } while ( z_ip2 );

    return;
}



/*
 * Local variables:
 * compile-command: "make -k && ./splines"
 * End:
 */
